
// =================================================     =====================================================
// =============================================================================
//           
// =============================================================================
#include <LPC2103.H>
#include <string.h>
#include "sect00.h" 
#include "mmcCommand.h"
#include "ls020.h"
#include "font.h"
#include "debug.h"


#define SD_CS_ON() 		(IOSET = (1 << 18))
#define SD_CS_OFF()		(IOCLR = (1 << 18))
const char Image11[];

unsigned long MBR_sector_BIOS_PB; //  ()    ( 0) -  446 + 08h)

//***********************************************   BPB (BIOS Parameter Block) ************************************************
unsigned long BPB_SectSize; //     (  BIOS PB -  0Bh)[  512]
unsigned char BPB_ClustSize;//     (  BIOS PB -  ODh)
unsigned long BPB_ResSect;  //     FAT#1 (  BIOS PB -  0Eh)
unsigned char BPB_FatCnt;   // -  FAT   (  BIOS PB -  10h)[  2]
unsigned long BPB_RootSize; //  32-    (  BIOS PB -  11h)( FAT16   512)
unsigned long BPB_FatSize;  //     FAT (  BIOS PB -  16h)
//*************************************************************************************************************************************

//**********************************************   *******************************************************************
unsigned long FAT_begin; //  FAT (  )
unsigned long Root_dir_begin; //    (  )
unsigned long Data_begin; //    ( 2)  
//unsigned char SectorBuffer[512]; //    
unsigned long MMC_size;
unsigned long F_FileSize;

// *************************************************************************************************************************************

unsigned char SectorBuffer[512];
//=====================================================================
//=====================================================================
char MMS_SPI(unsigned char lcd_data)
{ 
	SSPDR = lcd_data;
//	while ((SSPSR & (1 << 0)) == 0){;}
    delaySF_ms(0x01);
	return SSPDR; 
}
//========================================================================

 char MMC_Command(unsigned char cmd, unsigned long arg) 
{
        char r1,retry;
        retry=0;

        // 
        MMS_SPI(cmd|0x40);//
        MMS_SPI(arg>>24);//
        MMS_SPI(arg>>16);
        MMS_SPI(arg>>8);
        MMS_SPI(arg); 
        MMS_SPI(0x95);//      

        // ,     64 a,      0FF
        while((r1=MMS_SPI(0xff))==0xFF)if(retry++>64)break;

        // 
        return r1;
}
//=============================================================================

char MMC_SendCommand(unsigned char cmd, unsigned long arg) {
        char r1;
	//	highCS;
        SD_CS_OFF();
        r1=MMC_Command(cmd,arg);
        SD_CS_ON();
        MMS_SPI(0xFF);
		//lowCS;
        return r1; 
}

//=========================================================================

unsigned char MMC_Init(void) {
	
       unsigned char retry,r1;
        r1=retry=0;
		IODIR |= (1 << 18);		
	
        //    SPI
        do {
				SD_CS_ON();
                MMS_SPI(0xFF); 
                MMS_SPI(0xFF); 
                MMS_SPI(0xFF); 
                MMS_SPI(0xFF); 
                MMS_SPI(0xFF); 
                MMS_SPI(0xFF); 
                MMS_SPI(0xFF); 
                MMS_SPI(0xFF); 
                MMS_SPI(0xFF); 
                MMS_SPI(0xFF);               
                r1=MMC_SendCommand(MMC_GO_IDLE_STATE, 0);
                retry++;
                if(retry>100)return 100;
        } while(r1!=0x01);  


        // 
        retry=0;
        do { 
                r1=MMC_SendCommand(MMC_SEND_OP_COND,0);
//                delay_ms(1);
                retry++;
                if(retry>100)return r1;
        } while(r1);
//
        // CRC 
        r1=MMC_SendCommand(MMC_CRC_ON_OFF, 0);
        r1=MMC_SendCommand(MMC_SET_BLOCKLEN, 512);//   
//		//lowCS;
//		   SD_CS_OFF();
        return r1;
	 
}
//=========================================================================

unsigned char MMC_ReadSector(unsigned long sector) {
        char r1;
        int i;
//		highCS;
        SD_CS_OFF();// 
        //LED_ON(); //  

        r1=MMC_Command(MMC_READ_SINGLE_BLOCK, sector<<9);//  
        if(r1!=0)return r1;//      
		SD_CS_OFF();
        while(MMS_SPI(0xFF)!=MMC_STARTBLOCK_READ);//  

        for(i=0;i<512;i++) { //   
                SectorBuffer[i]=MMS_SPI(0xFF);
        } 

        // 2  CRC
        MMS_SPI(0xFF);
        MMS_SPI(0xFF);

        SD_CS_ON();// 
        MMS_SPI(0xFF);
		//lowCS;
        //LED_OFF(); //  
        return 0;
} 


//=====================================================================
//=====================================================================



unsigned long  F_StartCluster, //    (  .. )
	F_ClusterBuf, //    ,       ( 0    )
	Current_sector,
	F_Size; //   


extern unsigned char y_lcd;
unsigned char record;
extern unsigned char  i;
extern unsigned char attrib;
//extern char 


void Send(const char *Buff, unsigned char cnt) //    FLASH
{
		memcpy(SectorBuffer, Buff, 512);
}

unsigned char FAT_init(void) 
{
        unsigned char result;


//        //   ()  (MBR - Master Boot Record)
        result = MMC_ReadSector(0);
        if (result) return result; //    
//        
//        if (SectorBuffer[510] != 0x55 || SectorBuffer[511] != 0xAA)
//         {
////         lcdPrintText("Not formatted", 2,20, BLACK);
//         }      
//         

//		Send(Image22,12);

        BPB_SectSize = SectorBuffer[0x0c];  //     (  BIOS PB)[  512]
        BPB_SectSize <<=8;  
        BPB_SectSize += SectorBuffer[0x0b]; //
        //lcdPrintText("Sector size", 2,20, BLACK);
        //lcd_out_dec(120,20,BPB_SectSize,BLACK); // :    MBR
        
        BPB_ClustSize = SectorBuffer[0x0D]; //     (  BIOS PB)
        //lcdPrintText("Cluster size", 2,36, BLACK);
        //lcd_out_dec(120,36,BPB_ClustSize,BLACK);
        
        BPB_ResSect = SectorBuffer[ 0x0f];  //     FAT#1 (  BIOS PB) 
        BPB_ResSect <<=8;                   //
        BPB_ResSect += SectorBuffer[ 0x0e]; //

        //lcdPrintText("Reserv sector", 2,52, BLACK);
        //lcd_out_dec(120,52,BPB_ResSect,BLACK);
       
        BPB_FatCnt = SectorBuffer[0x10];    // -  FAT   (  BIOS PB)[  2]

        BPB_RootSize = SectorBuffer[0x12];  //  32-    ( FAT16   512)
        BPB_RootSize <<=8;                  //                                                  ( FAT32   0)
        BPB_RootSize += SectorBuffer[0x11]; //

        BPB_FatSize = SectorBuffer[0x17];   //     FAT (  BIOS PB)
        BPB_FatSize  <<=8;                  //
        BPB_FatSize  += SectorBuffer[0x16]; //
        
        //lcdPrintText("Fat Size", 2,68, BLACK);
        //lcd_out_dec(120,68,BPB_FatSize,BLACK);
        //     
        MMC_size = SectorBuffer[0x23];
        MMC_size <<=8;
        MMC_size+= SectorBuffer[0x22];
        MMC_size <<=8;
        MMC_size+= SectorBuffer[0x21];
        MMC_size <<=8;
        MMC_size+= SectorBuffer[0x20];
        MMC_size = MMC_size*512;
        MMC_size=MMC_size/1000000;
     	MBR_sector_BIOS_PB = 0;
       //   
        FAT_begin = MBR_sector_BIOS_PB + BPB_ResSect; //  FAT
        Root_dir_begin = FAT_begin + (BPB_FatSize * BPB_FatCnt); //   
        Data_begin = Root_dir_begin + (BPB_RootSize * 32 / 512); //    ( 2)
		 SD_CS_OFF();
		 //lowCS;
        return 0;
}


//===============================================================================================
//        FAT
//  :    
// ==============================================================================================
unsigned long  NextCluster(unsigned long cluster) 
{ 
        unsigned long tmp1, tmp2, tmp3, tmp4, tmp5;
        tmp1 = cluster / 256;    //    FAT (  FAT)
        tmp2 = FAT_begin + tmp1; //    
        tmp3 = cluster % 256;    //      (..  )
        tmp4 = 2 * tmp3;         //    
        MMC_ReadSector(tmp2);       
        tmp5 = SectorBuffer[tmp4+1]; //   
        tmp5 <<=8;                   //
        tmp5 += SectorBuffer[tmp4];  //
        F_ClusterBuf=0; //  0 ..        
        return tmp5; //    ()
}


////===========================
//unsigned char Readfirstsector(unsigned long cluster) 
//{
//        unsigned char result; 
//        Current_sector= (Data_begin+(cluster-2)*BPB_ClustSize);       //    ,    
//        return result;                                                              // BPB_ClustSize .
//}                                                                      //         
//                                                                      //      
//       
//unsigned long Get_file_size(void)
//        {
//    F_Size = SectorBuffer[(record*32)+0x1F];
//    F_Size <<= 8;
//    F_Size += SectorBuffer[(record*32)+0x1E];
//    F_Size <<= 8;
//    F_Size += SectorBuffer[(record*32)+0x1D];
//    F_Size <<= 8;
//    F_Size += SectorBuffer[(record*32)+0x1C]; 
//    return F_Size;
//        }
//
// unsigned long Get_Start_Cluster(void)
// {
//	F_StartCluster = SectorBuffer[(record*32)+0x15];     //    **   32
//	F_StartCluster <<= 8;
//	F_StartCluster += SectorBuffer[(record*32)+0x14];   //     **   32
//	F_StartCluster <<= 8;
//	F_StartCluster = SectorBuffer[(record*32)+0x1B];   //   
//	F_StartCluster <<= 8;
//	F_StartCluster += SectorBuffer[(record*32)+0x1A];  //   
//	        return F_StartCluster;
//}


char UniToASC(unsigned char data1,unsigned char data2)
{
    int chr;
//	unsigned char res1;
	if (data2 == 0) return(data1);
	else{
	  chr = data2<<8;
	  chr+=data1;
	  	chr -= 1025;
		if (chr < 86)
		return (tbUniToASC[chr]);
	}
  return (data1);
}
// ======================================================================================

void FindFiles(void)
{
char F_LName[64];
char attrib,i,setL;
int root_sector;
static unsigned char setLL;

for (root_sector = Root_dir_begin; root_sector <= (Root_dir_begin+32);root_sector++){
	MMC_ReadSector(root_sector);
    record =0;
	setLL = 0;
    while(SectorBuffer[record*32] !=0){
	if (SectorBuffer[record*32] != 0xe5) { //    -   ,  
	  attrib = SectorBuffer[(record*32)+11]; //  
	  if (attrib == 0x20 || attrib == 0x10 ){
	    F_StartCluster = SectorBuffer[(record*32)+27]<<8;
	    F_StartCluster += SectorBuffer[(record*32)+26];
		F_FileSize =  SectorBuffer[(record*32)+30]<<16;
		F_FileSize +=  SectorBuffer[(record*32)+29]<<8;
		F_FileSize += SectorBuffer[(record*32)+28];
		}
	   else if (attrib == 0x0F) {
		 setL = SectorBuffer[(record*32)];
		 setL &= 0x3f; 
		 if (setL > 0) setL--;
			 setL*=13;
			 F_LName[setL]    = UniToASC(SectorBuffer[(record*32)+1],SectorBuffer[(record*32)+2]);
			 F_LName[setL+1]  = UniToASC(SectorBuffer[(record*32)+3],SectorBuffer[(record*32)+4]);
			 F_LName[setL+2]  = UniToASC(SectorBuffer[(record*32)+5],SectorBuffer[(record*32)+6]);
			 F_LName[setL+3]  = UniToASC(SectorBuffer[(record*32)+7],SectorBuffer[(record*32)+8]);
			 F_LName[setL+4]  = UniToASC(SectorBuffer[(record*32)+9],SectorBuffer[(record*32)+10]);
			 F_LName[setL+5]  = UniToASC(SectorBuffer[(record*32)+14],SectorBuffer[(record*32)+15]);
			 F_LName[setL+6]  = UniToASC(SectorBuffer[(record*32)+16],SectorBuffer[(record*32)+17]);
			 F_LName[setL+7]  = UniToASC(SectorBuffer[(record*32)+18],SectorBuffer[(record*32)+19]);
			 F_LName[setL+8]  = UniToASC(SectorBuffer[(record*32)+20],SectorBuffer[(record*32)+21]);
			 F_LName[setL+9]  = UniToASC(SectorBuffer[(record*32)+22],SectorBuffer[(record*32)+23]);
			 F_LName[setL+10] = UniToASC(SectorBuffer[(record*32)+24],SectorBuffer[(record*32)+25]);
			 F_LName[setL+11] = UniToASC(SectorBuffer[(record*32)+28],SectorBuffer[(record*32)+29]);
			 F_LName[setL+12] = UniToASC(SectorBuffer[(record*32)+30],SectorBuffer[(record*32)+31]);
			 setLL += 13;
			 if (setL == 0){
			 debug(" \n\000");
			 for (i=0;i<setLL;i++){
			 if ((F_LName[i]==0xFF))  break; 
			 putchar1(F_LName[i]);
			 }
		 setLL = 0;
		 }
	   }
	}
	record++;
  }
 }
}
//======================================================================================================



        
  
